// ExoView.java


import java.nio.IntBuffer;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
//import javax.media.opengl.*;
import com.sun.opengl.util.BufferUtil;


public class ExoView implements GLEventListener
{
	
  /** Rendered 3D primitive */
  private ColorCube myBox = new ColorCube ();

  /** Half value of the field of view */
  private double fieldOfView = 1.0;
  private boolean fieldOfViewChanged = false;
  private static final double FIELD_OF_VIEW_INC = 1.01;
  private static final double FIELD_OF_VIEW_MIN = 0.5;
  private static final double FIELD_OF_VIEW_MAX = 10.0;

  /** azimuth angle (site) */
  private float azimuthAngle = 0.0f;
  private int azimuthMobility = 0;
  private static final float AZIMUTH_INC = 1.0f;

  /** height angle (elevation) */
  private float heightAngle = 0.0f;
  private int heightMobility = 0;
  private static final float HEIGHT_INC = 1.0f;
  
  Camera c = new Camera();

  /** Called by the drawable immediately after the OpenGL context is
    * initialized for the first time.
    * Implementation from GLEventListener. 
    * Used to perform one-time OpenGL initializations.
    * @param gLDrawable GLAutoDrawable object.
    */
  public void init (GLAutoDrawable gLDrawable)
  {
	  
	  
	 

	gLDrawable.addMouseMotionListener(c);
	gLDrawable.addKeyListener(c);
	
	
	
	
    final GL gl = gLDrawable.getGL ();
    gl.glShadeModel (GL.GL_FLAT);                  // No Smooth Shading
    gl.glClearColor (0.0f, 0.0f, 0.0f, 0.5f);      // Black Background
    gl.glClearDepth (1.0f);                        // Depth Buffer Setup
    gl.glEnable (GL.GL_DEPTH_TEST);                // Enables Depth Testing
    gl.glDepthFunc (GL.GL_LEQUAL);                 // Type Of Depth Testing
    gl.glEnable (GL.GL_CULL_FACE);                 // Face culling
    c.setPerspective(gl);
  }

  /** Called by the drawable to initiate OpenGL rendering by the client.
    * Implementation from GLEventListener. 
    * After all GLEventListeners have been notified of a display event,
    * the drawable will swap its buffers if necessary.
    * @param gLDrawable GLAutoDrawable object.
    */
  public void display (GLAutoDrawable gLDrawable)
  {
    final GL gl = gLDrawable.getGL ();
    
    final Camera c = new Camera();
    gl.glClearColor (0.0f, 0.0f, 0.0f, 0.5f); // Black Background
    if (fieldOfViewChanged)
    {
      IntBuffer vp = BufferUtil.newIntBuffer(4);
      gl.glGetIntegerv (GL.GL_VIEWPORT, vp);
      setProjection (gl, vp.get (2), vp.get (3));
      fieldOfViewChanged = false;
    }

    gl.glClear (GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
    gl.glLoadIdentity ();  // Reset The View
    
    
    c.setPerspective(gl);
    
    //gl.glTranslatef(0.f,0.f,-5.f);
    
    
    if (azimuthMobility != 0)
      azimuthAngle += AZIMUTH_INC * azimuthMobility;
    if (heightMobility != 0)
      heightAngle += HEIGHT_INC * heightMobility;

    
    
    
    
    gl.glRotatef (azimuthAngle, 0.0f, 1.0f, 0.0f);
    gl.glRotatef (heightAngle, 1.0f, 0.0f, 0.0f);
  
    myBox.draw (gl);
  }

  /** Called by the drawable during the first repaint after the component
    * has been resized.
    * Implementation from GLEventListener. 
    * Used to update the viewport and view volume appropriately,
    * for example by a call to GL.glViewport(int, int, int, int).
    * Note that for convenience the component has already called
    * GL.glViewport(int, int, int, int)(x, y, width, height) when this
    * method is called, so the client may not have to do anything.
    * @param gLDrawable GLAutoDrawable object.
    * @param x X Coordinate of the viewport area.
    * @param y Y coordinate of the viewport area.
    * @param width new width of the window.
    * @param height new height of the window.
    */
  public void reshape (GLAutoDrawable gLDrawable,
                       int x, int y, int width, int height)
  {
    final GL gl = gLDrawable.getGL ();

    if (height <= 0) // avoid a divide by zero error!
      height = 1;
    gl.glViewport (0, 0, width, height);
    setProjection (gl, width, height);
    gl.glLoadIdentity ();
  }

  /** Called when the display mode has been changed.
    * Implementation from GLEventListener. 
    * <B>!! CURRENTLY UNIMPLEMENTED IN JOGL !!</B>
    * @param gLDrawable GLAutoDrawable object.
    * @param modeChanged Indicates if the video mode has changed.
    * @param deviceChanged Indicates if the video device has changed.
    */
  public void displayChanged (GLAutoDrawable gLDrawable,
                              boolean modeChanged, boolean deviceChanged)
  {
  }

  /** Resets the projection matrix according to new viewport and
    * projection angle values.
    * @param gl OpenGL context.
    * @param width viewport width.
    * @param height viewport height.
    */ 
  private void setProjection (GL gl, int width, int height)
  {
    gl.glMatrixMode (GL.GL_PROJECTION);
    gl.glLoadIdentity ();
    gl.glOrtho (- fieldOfView, fieldOfView,
                - fieldOfView * height / width,
                fieldOfView * height / width, - 10.0, 10.0);
    gl.glMatrixMode (GL.GL_MODELVIEW);
  }

  /** Increases the projection field of view.
    */
  public void increaseFieldOfView ()
  {
    fieldOfView /= FIELD_OF_VIEW_INC;
    if (fieldOfView < FIELD_OF_VIEW_MIN) fieldOfView = FIELD_OF_VIEW_MIN;
    fieldOfViewChanged = true;
  }

  /** Decreases the projection field of view.
    */
  public void decreaseFieldOfView ()
  {
    fieldOfView *= FIELD_OF_VIEW_INC;
    if (fieldOfView > FIELD_OF_VIEW_MAX) fieldOfView = FIELD_OF_VIEW_MAX;
    fieldOfViewChanged = true;
  }

  /** Controls the primitive rotation in azimuth.
    * @param control rotation control :
    *   fixed if set to 0,
    *   counter-clockwise if set to 1,
    *   clockwise if set to -1.
    */ 
  public void rotateInAzimuth (int control)
  {
    azimuthMobility = control;
  }

  /** Controls the primitive rotation in elevation.
    * @param control rotation control :
    *   fixed if set to 0,
    *   counter-clockwise if set to 1,
    *   clockwise if set to -1.
    */ 
  public void rotateInHeight (int control)
  {
    heightMobility = control;
  }
}
